/*-- 

 Copyright (C) 2001 Anthony Eden.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions, and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions, and the disclaimer that follows 
    these conditions in the documentation and/or other materials 
    provided with the distribution.

 3. The name "JPublish" must not be used to endorse or promote products
    derived from this software without prior written permission.  For
    written permission, please contact me@anthonyeden.com.
 
 4. Products derived from this software may not be called "JPublish", nor
    may "JPublish" appear in their name, without prior written permission
    from Anthony Eden (me@anthonyeden.com).
 
 In addition, I request (but do not require) that you include in the 
 end-user documentation provided with the redistribution and/or in the 
 software itself an acknowledgement equivalent to the following:
     "This product includes software developed by
      Anthony Eden (http://www.anthonyeden.com/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.

 For more information on JPublish, please see <http://www.jpublish.org/>.
 
 */

package org.jpublish;

import java.io.File;

import java.io.InputStream;
import java.io.FileInputStream;

import java.util.List;
import java.util.Map;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.input.SAXBuilder;

import org.apache.log4j.Category;

import org.formproc.FormManager;

import org.jpublish.util.FileMonitor;
import org.jpublish.action.Action;
import org.jpublish.action.ScriptAction;
import org.jpublish.action.PathAction;
import org.jpublish.action.ActionManager;

/**	This class contains configuration information for a particular
	site.  Pages are loaded and actions are executed within the
	context of the site and have access to all of the methods within
	this class.
	
	<p>Instances of the SiteContext class will also reload themselves
	automatically when the underlying configuration file changes.</p>
	
	@author Anthony Eden
*/

public class SiteContext implements XMLConstants, Reloadable{

	/**	Construct a new SiteContext using the given File to load the
		context's configuration.
		
		@param configurationFile The configuration File
	*/

	public SiteContext(File configurationFile){
		this.configurationFile = configurationFile;
		
		repositories = new ArrayList();
		staticResourceLoader = new StaticResourceLoader();
		
		reload();
		
		FileMonitor fileMonitor = new FileMonitor(this.configurationFile, 5000, this);
		fileMonitor.startMonitor();
	}

	public File getPageRoot(){
		return pageRoot;
	}
	
	public void setPageRoot(File pageRoot){
		this.pageRoot = pageRoot;
	}
	
	public void setPageRoot(String pageRoot){
		if(pageRoot != null){
			setPageRoot(new File(pageRoot));
		}
	}
	
	public File getTemplateRoot(){
		return templateRoot;
	}
	
	public void setTemplateRoot(File templateRoot){
		this.templateRoot = templateRoot;
	}
	
	public void setTemplateRoot(String templateRoot){
		if(templateRoot != null){
			setTemplateRoot(new File(templateRoot));
		}
	}
	
	public File getActionRoot(){
		return actionRoot;
	}
	
	public void setActionRoot(File actionRoot){
		this.actionRoot = actionRoot;
	}
	
	public void setActionRoot(String actionRoot){
		if(actionRoot != null){
			setActionRoot(new File(actionRoot));
		}
	}
	
	public File getStaticRoot(){
		return staticRoot;
	}
	
	public void setStaticRoot(File staticRoot){
		this.staticRoot = staticRoot;
	}
	
	public void setStaticRoot(String staticRoot){
		if(staticRoot != null){
			setStaticRoot(new File(staticRoot));
		}
	}
	
	public StaticResourceLoader getStaticResourceLoader(){
		return staticResourceLoader;
	}
	
	public File getWebInfPath(){
		return webInfPath;
	}
	
	public void setWebInfPath(File webInfPath){
		this.webInfPath = webInfPath;
	}
	
	/**	Get the action identifier which is used to trigger parameter actions.
		The default value is 'action'.
	
		@return The action identifier
	*/
	
	public String getActionIdentifier(){
		return actionIdentifier;
	}
	
	/**	Set the action identifier.  If this method is invoked with a null
		argument then the action identifier will be reset to the default
		value.
		
		@param actionIdentifier The new action identifer or null to reset
	*/
	
	public synchronized void setActionIdentifier(String actionIdentifier){
		if(actionIdentifier == null){
			this.actionIdentifier = DEFAULT_ACTION_INDENTIFIER;
		} else {
			this.actionIdentifier = actionIdentifier;
		}
	}
	
	public String getDefaultPage(){
		return defaultPage;
	}
	
	public void setDefaultPage(String defaultPage){
		this.defaultPage = defaultPage;
	}
	
	public String getDefaultTemplate(){
		return defaultTemplate;
	}
	
	public void setDefaultTemplate(String defaultTemplate){
		this.defaultTemplate = defaultTemplate;
	}
	
	public List getRepositories(){
		return repositories;
	}
	
	/**	Get the site's ActionManager.
	
		@return The ActionManager
		@see org.jpublish.action.ActionManager
	*/
	
	public ActionManager getActionManager(){
		return actionManager;
	}
	
	/**	Get the site's PageManager.
	
		@return The PageManager
		@see org.jpublish.PageManager
	*/
	
	public PageManager getPageManager(){
		return pageManager;
	}
	
	/**	Get the site's FormManager.  If no FormManager is specified then
		this method will return null.
	
		@return The FormManager
		@see org.formproc.FormManager
	*/
	
	public FormManager getFormManager(){
		return formManager;
	}
	
	/**	Reload the site configuration. */
	
	public void reload(){
		try{
			log.info("Loading site configuration.");
			loadConfiguration();
			log.info("Configuration loaded.");
		} catch(Exception e){
			e.printStackTrace();
		}
	}
	
	/**	Load the site configuration from the given InputStream.  The
		InputStream must be attached to an XML document.
		
		@param in The InputStream
		@throws Exception
	*/

	public void load(InputStream in) throws Exception{
		actionManager = new ActionManager(this);
		pageManager = new PageManager(this);
	
		SAXBuilder builder = new SAXBuilder();
		Document document = builder.build(in);
		Element rootElement = document.getRootElement();
		
		setPageRoot(rootElement.getChildTextTrim("page-root", NS));
		setTemplateRoot(rootElement.getChildTextTrim("template-root", NS));
		setActionRoot(rootElement.getChildTextTrim("action-root", NS));
		setStaticRoot(rootElement.getChildTextTrim("static-root", NS));
		
		createFormManager(rootElement.getChildTextTrim("formproc-config", NS));
		
		setActionIdentifier(rootElement.getChildTextTrim("action-identifier", NS));
		
		setDefaultPage(rootElement.getChildTextTrim("default-page", NS));
		setDefaultTemplate(rootElement.getChildTextTrim("default-template", NS));
		
		Iterator repositoryElements = rootElement.getChildren("repository", NS).iterator();
		while(repositoryElements.hasNext()){
			Element repositoryElement = (Element)repositoryElements.next();
			String className = repositoryElement.getAttributeValue("classname");
			Repository repository = (Repository)Class.forName(className).newInstance();
			((Configurable)repository).loadConfiguration(repositoryElement);
			getRepositories().add(repository);
		}
		
		Map definedActions = actionManager.getDefinedActions();
		Iterator defineActionElements = rootElement.getChildren("define-action", NS).iterator();
		while(defineActionElements.hasNext()){
			Element defineActionElement = (Element)defineActionElements.next();
			String name = defineActionElement.getAttributeValue("name");
			String className = defineActionElement.getAttributeValue("classname");
			Action action = (Action)Class.forName(className).newInstance();
			definedActions.put(name, action);
		}
		
		List globalActions = actionManager.getGlobalActions();
		Iterator globalActionElements = rootElement.getChildren("global-action", NS).iterator();
		while(globalActionElements.hasNext()){
			Element globalActionElement = (Element)globalActionElements.next();
			String name = globalActionElement.getAttributeValue("name");
			if(definedActions.containsKey(name)){
				globalActions.add(definedActions.get(name));
			} else {
				File script = new File(getActionRoot(), name);
				globalActions.add(new ScriptAction(this, script));
			}
		}
		
		List pathActions = actionManager.getPathActions();
		Iterator pathActionElements = rootElement.getChildren("path-action", NS).iterator();
		while(pathActionElements.hasNext()){
			Element pathActionElement = (Element)pathActionElements.next();
			String name = pathActionElement.getAttributeValue("name");
			String path = pathActionElement.getAttributeValue("path");
			if(definedActions.containsKey(name)){
				pathActions.add(new PathAction(path, (Action)definedActions.get(name)));
			} else {
				File script = new File(getActionRoot(), name);
				pathActions.add(new PathAction(path, new ScriptAction(this, script)));
			}
		}
	}
	
	// private methods
	
	private void loadConfiguration() throws Exception{
		InputStream configIn = null;
		
		try{
			configIn = new FileInputStream(configurationFile);
			load(configIn);
			configIn.close();
		} catch(Exception e){
			throw e;
		} finally {
			if(configIn != null){
				try{
					configIn.close();
				} catch(Exception e){
					// do nothing
				}
			}
		}
	}
	
	private void createFormManager(String configFile){
		if(configFile != null){
			formManager = new FormManager(new File(configFile));
		}
	}
	
	private static final String DEFAULT_ACTION_INDENTIFIER = "action";
	
	private static Category log = Category.getInstance(SiteContext.class.getName());
	
	private File configurationFile;
	
	private File pageRoot;
	private File templateRoot;
	private File actionRoot;
	private File staticRoot;
	private File webInfPath;
	private String actionIdentifier;
	private String defaultPage = "index.html";
	private String defaultTemplate = "basic";
	private List repositories;
	private Map definedActions;
	private List globalActions;
	private List pathActions;
	
	private PageManager pageManager;
	private ActionManager actionManager;
	private StaticResourceLoader staticResourceLoader;
	private FormManager formManager;

}
