/*-- 

 Copyright (C) 2001 Anthony Eden.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions, and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions, and the disclaimer that follows 
    these conditions in the documentation and/or other materials 
    provided with the distribution.

 3. The name "JPublish" must not be used to endorse or promote products
    derived from this software without prior written permission.  For
    written permission, please contact me@anthonyeden.com.
 
 4. Products derived from this software may not be called "JPublish", nor
    may "JPublish" appear in their name, without prior written permission
    from Anthony Eden (me@anthonyeden.com).
 
 In addition, I request (but do not require) that you include in the 
 end-user documentation provided with the redistribution and/or in the 
 software itself an acknowledgement equivalent to the following:
     "This product includes software developed by
      Anthony Eden (http://www.anthonyeden.com/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.

 For more information on JPublish, please see <http://www.jpublish.org/>.
 
 */

package org.jpublish.action;

import java.io.File;
import java.util.Map;
import java.util.HashMap;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

import org.apache.log4j.Category;
import org.apache.velocity.VelocityContext;

import org.jpublish.SiteContext;
import org.jpublish.util.PathUtilities;

public class ActionManager{

	public ActionManager(SiteContext siteContext){
		this.siteContext = siteContext;
		this.definedActions = new HashMap();
		this.globalActions = new ArrayList();
		this.pathActions = new ArrayList();
	}
	
	public Map getDefinedActions(){
		return definedActions;
	}

	public void addAction(String name, Action action){
		definedActions.put(name, action);
	}
	
	public void removeAction(String name){
		definedActions.remove(name);
	}
	
	public List getGlobalActions(){
		return globalActions;
	}
	
	public List getPathActions(){
		return pathActions;
	}
	
	public String executeGlobalActions(VelocityContext context) throws Exception{
		Iterator actions = getGlobalActions().iterator();
		while(actions.hasNext()){
			Action action = (Action)actions.next();
			action.execute(context);
			
			String redirect = (String)context.get("redirect");
			if(redirect != null){
				return redirect;
			}
		}
		return null;
	}
	
	public String executePathActions(String path, VelocityContext context) throws Exception{
		Iterator actions = getPathActions().iterator();
		while(actions.hasNext()){
			PathAction action = (PathAction)actions.next();
			if(PathUtilities.match(path, action.getPath())){
				action.execute(context);
				
				String redirect = (String)context.get("redirect");
				if(redirect != null){
					return redirect;
				}
			}
		}
		return null;
	}
	
	public String execute(String name, VelocityContext context) throws Exception{	
		Action action = findAction(name);
		if(action != null){
			action.execute(context);
			
			String redirect = (String)context.get("redirect");
			if(redirect != null){
				return redirect;
			}
		}
		return null;
	}

	public Action findAction(String name){
		// look in registered classes first
		log.debug("Looking for registered action.");
		Action action = (Action)definedActions.get(name);
		if(action != null){
			log.debug("Registered action found.");
			return action;
		}
		
		// look in the action directory for scripts
		log.debug("Looking for actions in action root.");
		File actionRoot = siteContext.getActionRoot();
		File[] files = actionRoot.listFiles();
		for(int i = 0; i < files.length; i++){
			if(files[i].getName().equals(name)){
				log.debug("Action found [" + files[i] + "]");
				action = new ScriptAction(siteContext, files[i]);
				return action;
			}
		}
		
		return null;
	}
	
	private static Category log = Category.getInstance(ActionManager.class.getName());
	
	private Map definedActions;
	private List globalActions;
	private List pathActions;
	private SiteContext siteContext;

}
