/*-- 

 Copyright (C) 2001 Anthony Eden.
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions, and the following disclaimer.
 
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions, and the disclaimer that follows 
    these conditions in the documentation and/or other materials 
    provided with the distribution.

 3. The name "JPublish" must not be used to endorse or promote products
    derived from this software without prior written permission.  For
    written permission, please contact me@anthonyeden.com.
 
 4. Products derived from this software may not be called "JPublish", nor
    may "JPublish" appear in their name, without prior written permission
    from Anthony Eden (me@anthonyeden.com).
 
 In addition, I request (but do not require) that you include in the 
 end-user documentation provided with the redistribution and/or in the 
 software itself an acknowledgement equivalent to the following:
     "This product includes software developed by
      Anthony Eden (http://www.anthonyeden.com/)."

 THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR(S) BE LIABLE FOR ANY DIRECT, 
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) 
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 POSSIBILITY OF SUCH DAMAGE.

 For more information on JPublish, please see <http://www.jpublish.org/>.
 
 */

package org.jpublish.servlet;

import java.io.File;
import java.io.IOException;
import java.io.Writer;
import java.io.FileInputStream;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.FileNotFoundException;
import java.util.Iterator;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.apache.log4j.Category;
import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.PropertyConfigurator;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.app.HttpServletRequestWrap;
import org.apache.velocity.app.HttpServletResponseWrap;

import org.jpublish.Page;
import org.jpublish.Repository;
import org.jpublish.RepositoryWrapper;
import org.jpublish.SiteContext;
import org.jpublish.util.LogSystem;
import org.jpublish.util.URLUtilities;
import org.jpublish.action.ActionManager;

/**	This class is the entry point for requests to the JPublish framework.
	The servlet should be configured to handler all requests for its context.
	In addition, a init-parameter called config must be included in the
	<code>web.xml</code> file.  The value of this parameter must be the
	full path to the site's configuration file.
	
	@author Anthony Eden
*/

public class JPublishServlet extends HttpServlet{

	/**	Initialize the servlet.
	
		@param servletConfig The Servlet configuration
		@throws ServletException
	*/

	public void init(ServletConfig servletConfig) throws 
		ServletException
	{
		super.init(servletConfig);
	
		// find the WEB-INF root
		String rootPath = getServletContext().getRealPath("/");
        File webInfPath = new File(rootPath, "WEB-INF");
		
		// check for a log.properties init
		String logProperties = servletConfig.getInitParameter("log.properties");
		LogSystem logSystem = new LogSystem(logProperties, webInfPath);
		
		// initialize velocity
		try{
			Velocity.init();
		} catch(Exception e){
			throw new ServletException(e);
		}
		
		File webLibPath = new File(webInfPath, "lib");
		File webClassPath = new File(webInfPath, "classes");
	
		// expand the class path for WEB-INF classes and archives	
		StringBuffer classPath = new StringBuffer();
		classPath.append(System.getProperty("java.class.path"));
		
		if(webClassPath.exists()){
			classPath.append(webClassPath);
		}
		
		File[] files = webLibPath.listFiles();
		for(int i = 0; i < files.length; i++){
			if(files[i].getName().toLowerCase().endsWith(".jar") ||
			   files[i].getName().toLowerCase().endsWith(".zip"))
			{
				classPath.append(System.getProperty("path.separator"));
				classPath.append(files[i]);
			}
		}
		
		System.setProperty("java.class.path", classPath.toString());
	
		// load the site context from the config file
		File configurationFile = new File(servletConfig.getInitParameter("config"));
		siteContext = new SiteContext(configurationFile);
		siteContext.setWebInfPath(webInfPath);
		//siteContext.initScriptingLanguages();
	}
	
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws
		ServletException, IOException
	{
		doPost(request, response);
	}

	public void doPost(HttpServletRequest request, HttpServletResponse response) throws 
		ServletException, IOException
	{
	
		ActionManager actionManager = siteContext.getActionManager();
		
		HttpSession session = request.getSession(true);
	
		String pathInfo = request.getPathInfo();
		log.debug("Path info: " + pathInfo);
		
		// get the real path
		String path = getRealPath(pathInfo);
		log.debug("Real path: " + path);
	
		VelocityContext vc = new VelocityContext();
		vc.put("request", new HttpServletRequestWrap(request));
		vc.put("response", new HttpServletResponseWrap(response));
		vc.put("session", new HttpSessionWrap(session));
		vc.put("url_util", new URLUtilities(request));
		
		Object testValue = session.getAttribute("test");
		if(testValue == null){
			log.debug("Placing test value in session.");
			session.setAttribute("test", "test value");
		} else {
			log.debug("Got test value from session.");
		}
	
		try{
			// execute the global actions
			log.debug("Executing global actions.");
			if(optionalRedirect(actionManager.executeGlobalActions(vc), 
				path, response))
			{
				return;
			}
			
			// execute path actions
			log.debug("Executing path actions.");
			if(optionalRedirect(actionManager.executePathActions(
				pathInfo, vc), path, response))
			{
				return;
			}
		
			// execute parameter actions
			log.debug("Executing parameter actions.");
			String[] actionNames = request.getParameterValues(siteContext.getActionIdentifier());
			if(actionNames != null){
				for(int i = 0; i < actionNames.length; i++){
					if(optionalRedirect(actionManager.execute(
						actionNames[i], vc), path, response))
					{
						return;
					}
				}
			}
			
			// if the page is static
			File file = new File(siteContext.getStaticRoot(), path);
			log.debug("Checking for static file: " + file);
			if(file.exists()){
				OutputStream out = response.getOutputStream();
				siteContext.getStaticResourceLoader().load(file, out);
				return;
			}
	
			// load the page
			log.debug("Loading the page.");
			Page page = siteContext.getPageManager().getPage(path);
			vc.put("page", page);
			
			// execute the page actions
			log.debug("Executing page actions.");
			if(optionalRedirect(page.executeActions(vc), path, response))
			{
				return;
			}
			
			// add the repositories to the VelocityContext
			Iterator repositories = siteContext.getRepositories().iterator();
			while(repositories.hasNext()){
				Repository repository = (Repository)repositories.next();
				log.debug("Adding " + repository.getClass().getName() + " as " + repository.getName());
				vc.put(repository.getName(), new RepositoryWrapper(repository, vc));
			}
			
			// get the template file
			File templateFile = new File(siteContext.getTemplateRoot(), 
				page.getFullTemplateName());
			
			// merge
			Writer out = response.getWriter();
			log.debug("Merging with template " + templateFile);
			Velocity.evaluate(vc, out, path, new FileInputStream(templateFile));
			log.debug("Merge complete.");
			
			out.flush();
			out.close();
		} catch(FileNotFoundException e){
			response.sendError(HttpServletResponse.SC_NOT_FOUND, e.getMessage());
		} catch(Exception e){
			throw new ServletException(e);
		}
	}
	
	private String getRealPath(String path){
		if(path == null){
			path = "";
		}
		
		if(path.lastIndexOf(".") == -1){
			if(!path.endsWith("/")){
				path = path + "/";
			}
			path = path + siteContext.getDefaultPage();
		}
		
		return path;
	}
	
	private boolean optionalRedirect(String redirect, String path, 
	HttpServletResponse response) throws IOException{
		if(redirect == null){
			return false;
		}
		
		if(redirect.lastIndexOf(".") == -1){
			response.sendRedirect(redirect + 
				path.substring(path.lastIndexOf(".")));
			return true;
		} else {
			response.sendRedirect(redirect);
			return true;
		}
	}
	
	private static Category log = Category.getInstance(JPublishServlet.class.getName());
	
	private SiteContext siteContext;

}
